﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace DragSample
{
    /// <summary>
    /// MainWindow.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// 드레그를 시작한 마우스 좌표;
        /// </summary>
        Point prePosition;
        /// <summary>
        /// 현재 그려지는 사각형
        /// </summary>
        Rectangle currentRect;

        public MainWindow()
        {
            InitializeComponent();
            //이벤트 핸들러 생성
            this.root.MouseLeftButtonDown += new MouseButtonEventHandler(root_MouseLeftButtonDown);
            this.root.MouseMove += new MouseEventHandler(root_MouseMove);
            this.root.MouseLeftButtonUp += new MouseButtonEventHandler(root_MouseLeftButtonUp);
            
            // 좌표를 나타내는 TextBlock을 최상위로 ...
            Grid.SetZIndex(this.tbPosition, 99999);
        }

        /// <summary>
        /// 마우스 클릭 해제 이벤트 핸들러
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void root_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            //마우스 캡춰를 제거한다.
            this.root.ReleaseMouseCapture();
            SetRectangleProperty();
            currentRect = null;
        }

        private void SetRectangleProperty()
        {
            //사각형의 투명도를 100% 로 설정
            currentRect.Opacity = 1;
            //사각형의 색상을 지정
            currentRect.Fill = new SolidColorBrush(Colors.LightYellow);
            //사각형의 테두리를 선으로 지정
            currentRect.StrokeDashArray = new DoubleCollection(); ;
        }

        /// <summary>
        /// 마우스 이동 이벤트 핸들러
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void root_MouseMove(object sender, MouseEventArgs e)
        {
            
            //현재 이동한 마우스의 좌표를 얻어온다
            Point currnetPosition = e.GetPosition(this.root);
            //좌표를 표시한다.
            this.tbPosition.Text = string.Format("마우스 좌표 : [{0},{1}]", currnetPosition.X, currnetPosition.Y);
            //마우스 왼쪽 버튼이 눌려있으면
            if (e.MouseDevice.LeftButton == MouseButtonState.Pressed)
            {
                if (currentRect != null)
                {
                    //사각형이 나타날 기준점을 설정한다.
                    double left = prePosition.X;
                    double top = prePosition.Y;
                    //마우스의 위치에 따라 적절히 기준점을 변경한다.
                    if (prePosition.X > currnetPosition.X)
                    {
                        left = currnetPosition.X;
                    }
                    if (prePosition.Y > currnetPosition.Y)
                    {
                        top = currnetPosition.Y;
                    }
                    //사각형의 위치 기준점(Margin)을 설정한다
                    currentRect.Margin = new Thickness(left, top, 0, 0);
                    //사각형의 크기를 설정한다. 음수가 나올 수 없으므로 절대값을 취해준다.
                    currentRect.Width = Math.Abs(prePosition.X - currnetPosition.X);
                    currentRect.Height = Math.Abs(prePosition.Y - currnetPosition.Y);
                }
            }
        }

        /// <summary>
        /// 마우스 왼쪽 버튼 클릭 이벤트 핸들러
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void root_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //마우스의 좌표를 저장한다.
            prePosition = e.GetPosition(this.root);
            //마우스가 Grid밖으로 나가도 위치를 알 수 있도록 마우스 이벤트를 캡처한다.
            this.root.CaptureMouse();
            if (currentRect == null)
            {
                //사각형을 생성한다.
                CreteRectangle();               
            }
        }

        private void CreteRectangle()
        {
            
            currentRect = new Rectangle();
            currentRect.Stroke = new SolidColorBrush(Colors.DarkGreen);
            currentRect.StrokeThickness = 2;
            currentRect.Opacity = 0.7;
            //사각형을 그리는 동안은 테두리를 Dash 스타일로 설정한다.
            DoubleCollection dashSize = new DoubleCollection();
            dashSize.Add(1);
            dashSize.Add(1);
            currentRect.StrokeDashArray = dashSize;
            //사각형의 정렬 기준을 설정한다.
            currentRect.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
            currentRect.VerticalAlignment = System.Windows.VerticalAlignment.Top;
            //그리드에 추가한다.
            this.root.Children.Add(currentRect);
        }
    }
}
